@@ -1,9 +1,9 @@
-import alembic
-from alembic.operations import Operations
-from alembic.migration import MigrationContext
-from alembic import util
 from contextlib import contextmanager
 
+from .operations import Operations
+from .migration import MigrationContext
+from . import util
+
 class EnvironmentContext(object):
     """Represent the state made available to an ``env.py`` script.
 
@@ -99,12 +99,14 @@ class EnvironmentContext(object):
         be made available as ``from alembic import context``.
 
         """
-        alembic.context._install_proxy(self)
+        from .context import _install_proxy
+        _install_proxy(self)
         return self
 
     def __exit__(self, *arg, **kw):
-        alembic.context._remove_proxy()
-        alembic.op._remove_proxy()
+        from . import context, op
+        context._remove_proxy()
+        op._remove_proxy()
 
     def is_offline_mode(self):
         """Return True if the current migrations environment
@@ -195,9 +197,62 @@ class EnvironmentContext(object):
         This function does not require that the :class:`.MigrationContext`
         has been configured.
 
+        .. seealso::
+
+            :meth:`.EnvironmentContext.get_x_argument` - a newer and more
+            open ended system of extending ``env.py`` scripts via the command
+            line.
+
         """
         return self.context_opts.get('tag', None)
 
+    def get_x_argument(self, as_dictionary=False):
+        """Return the value(s) passed for the ``-x`` argument, if any.
+
+        The ``-x`` argument is an open ended flag that allows any user-defined
+        value or values to be passed on the command line, then available
+        here for consumption by a custom ``env.py`` script.
+
+        The return value is a list, returned directly from the ``argparse``
+        structure.  If ``as_dictionary=True`` is passed, the ``x`` arguments
+        are parsed using ``key=value`` format into a dictionary that is
+        then returned.
+
+        For example, to support passing a database URL on the command line,
+        the standard ``env.py`` script can be modified like this::
+
+            cmd_line_url = context.get_x_argument(as_dictionary=True).get('dbname')
+            if cmd_line_url:
+                engine = create_engine(cmd_line_url)
+            else:
+                engine = engine_from_config(
+                        config.get_section(config.config_ini_section),
+                        prefix='sqlalchemy.',
+                        poolclass=pool.NullPool)
+
+        This then takes effect by running the ``alembic`` script as::
+
+            alembic -x dbname=postgresql://user:pass@host/dbname upgrade head
+
+        This function does not require that the :class:`.MigrationContext`
+        has been configured.
+
+        .. versionadded:: 0.6.0
+
+        .. seealso::
+
+            :meth:`.EnvironmentContext.get_tag_argument`
+
+            :attr:`.Config.cmd_opts`
+
+        """
+        value = self.config.cmd_opts.x or []
+        if as_dictionary:
+            value = dict(
+                        arg.split('=', 1) for arg in value
+                    )
+        return value
+
     def configure(self,
             connection=None,
             url=None,
@@ -209,12 +264,16 @@ class EnvironmentContext(object):
             template_args=None,
             target_metadata=None,
             include_symbol=None,
+            include_object=None,
+            include_schemas=False,
             compare_type=False,
             compare_server_default=False,
+            render_item=None,
             upgrade_token="upgrades",
             downgrade_token="downgrades",
             alembic_module_prefix="op.",
             sqlalchemy_module_prefix="sa.",
+            user_module_prefix=None,
             **kw
         ):
         """Configure a :class:`.MigrationContext` within this
@@ -236,18 +295,18 @@ class EnvironmentContext(object):
         If the :meth:`.is_offline_mode` function returns ``True``,
         then no connection is needed here.  Otherwise, the
         ``connection`` parameter should be present as an
-        instance of :class:`sqlalchemy.engine.base.Connection`.
+        instance of :class:`sqlalchemy.engine.Connection`.
 
         This function is typically called from the ``env.py``
         script within a migration environment.  It can be called
         multiple times for an invocation.  The most recent
-        :class:`~sqlalchemy.engine.base.Connection`
+        :class:`~sqlalchemy.engine.Connection`
         for which it was called is the one that will be operated upon
         by the next call to :meth:`.run_migrations`.
 
         General parameters:
 
-        :param connection: a :class:`~sqlalchemy.engine.base.Connection`
+        :param connection: a :class:`~sqlalchemy.engine.Connection`
          to use
          for SQL execution in "online" mode.  When present, is also
          used to determine the type of dialect in use.
@@ -271,6 +330,11 @@ class EnvironmentContext(object):
          the :class:`.Config`
          object.  The value here overrides that of the :class:`.Config`
          object.
+        :param output_encoding: when using ``--sql`` to generate SQL
+         scripts, apply this encoding to the string output.
+
+         .. versionadded:: 0.5.0
+
         :param starting_rev: Override the "starting revision" argument
          when using ``--sql`` mode.
         :param tag: a string tag for usage by custom ``env.py`` scripts.
@@ -280,9 +344,16 @@ class EnvironmentContext(object):
          running the "revision" command.   Note that the script environment
          is only run within the "revision" command if the --autogenerate
          option is used, or if the option "revision_environment=true"
-         is present in the alembic.ini file.  New in 0.3.3.
+         is present in the alembic.ini file.
+
+         .. versionadded:: 0.3.3
+
         :param version_table: The name of the Alembic version table.
          The default is ``'alembic_version'``.
+        :param version_table_schema: Optional schema to place version
+         table within.
+
+         .. versionadded:: 0.5.0
 
         Parameters specific to the autogenerate feature, when
         ``alembic revision`` is run with the ``--autogenerate`` feature:
@@ -292,7 +363,7 @@ class EnvironmentContext(object):
          will be consulted during autogeneration.  The tables present
          will be compared against
          what is locally available on the target
-         :class:`~sqlalchemy.engine.base.Connection`
+         :class:`~sqlalchemy.engine.Connection`
          to produce candidate upgrade/downgrade operations.
 
         :param compare_type: Indicates type comparison behavior during
@@ -312,16 +383,26 @@ class EnvironmentContext(object):
                 # return True if the types are different,
                 # False if not, or None to allow the default implementation
                 # to compare these types
-                pass
+                return None
 
-         ``inspected_column`` is a dictionary structure as returned by
-         :meth:`sqlalchemy.engine.reflection.Inspector.get_columns`, whereas
+            context.configure(
+                # ...
+                compare_type = my_compare_type
+            )
+
+
+         ``inspected_column`` is a :class:`sqlalchemy.schema.Column` as returned by
+         :meth:`sqlalchemy.engine.reflection.Inspector.reflecttable`, whereas
          ``metadata_column`` is a :class:`sqlalchemy.schema.Column` from
          the local model environment.
 
          A return value of ``None`` indicates to allow default type
          comparison to proceed.
 
+         .. seealso::
+
+            :paramref:`.EnvironmentContext.configure.compare_server_default`
+
         :param compare_server_default: Indicates server default comparison
          behavior during
          an autogenerate operation.  Defaults to ``False`` which disables
@@ -343,7 +424,12 @@ class EnvironmentContext(object):
                 # return True if the defaults are different,
                 # False if not, or None to allow the default implementation
                 # to compare these defaults
-                pass
+                return None
+
+            context.configure(
+                # ...
+                compare_server_default = my_compare_server_default
+            )
 
          ``inspected_column`` is a dictionary structure as returned by
          :meth:`sqlalchemy.engine.reflection.Inspector.get_columns`, whereas
@@ -356,12 +442,70 @@ class EnvironmentContext(object):
          execute
          the two defaults on the database side to compare for equivalence.
 
-        :param include_symbol: A callable function which, given a table name
-         and optional schema name, returns ``True`` or ``False``, indicating
-         if the given table should be considered in the autogenerate sweep.
+         .. seealso::
+
+            :paramref:`.EnvironmentContext.configure.compare_type`
+
+        :param include_object: A callable function which is given
+         the chance to return ``True`` or ``False`` for any object,
+         indicating if the given object should be considered in the
+         autogenerate sweep.
+
+         The function accepts the following positional arguments:
+
+         * ``object``: a :class:`~sqlalchemy.schema.SchemaItem` object such as a
+           :class:`~sqlalchemy.schema.Table` or :class:`~sqlalchemy.schema.Column`
+           object
+         * ``name``: the name of the object. This is typically available
+           via ``object.name``.
+         * ``type``: a string describing the type of object; currently
+           ``"table"`` or ``"column"``
+         * ``reflected``: ``True`` if the given object was produced based on
+           table reflection, ``False`` if it's from a local :class:`.MetaData`
+           object.
+         * ``compare_to``: the object being compared against, if available,
+           else ``None``.
+
          E.g.::
 
-            def include_symbol(tablename, schema=None):
+            def include_object(object, name, type_, reflected, compare_to):
+                if (type_ == "column" and
+                    not reflected and
+                    object.info.get("skip_autogenerate", False)):
+                    return False
+                else:
+                    return True
+
+            context.configure(
+                # ...
+                include_object = include_object
+            )
+
+         :paramref:`.EnvironmentContext.configure.include_object` can also
+         be used to filter on specific schemas to include or omit, when
+         the :paramref:`.EnvironmentContext.configure.include_schemas`
+         flag is set to ``True``.   The :attr:`.Table.schema` attribute
+         on each :class:`.Table` object reflected will indicate the name of the
+         schema from which the :class:`.Table` originates.
+
+         .. versionadded:: 0.6.0
+
+         .. seealso::
+
+            :paramref:`.EnvironmentContext.configure.include_schemas`
+
+        :param include_symbol: A callable function which, given a table name
+         and schema name (may be ``None``), returns ``True`` or ``False``, indicating
+         if the given table should be considered in the autogenerate sweep.
+
+         .. deprecated:: 0.6.0 :paramref:`.EnvironmentContext.configure.include_symbol`
+            is superceded by the more generic
+            :paramref:`.EnvironmentContext.configure.include_object`
+            parameter.
+
+         E.g.::
+
+            def include_symbol(tablename, schema):
                 return tablename not in ("skip_table_one", "skip_table_two")
 
             context.configure(
@@ -369,7 +513,57 @@ class EnvironmentContext(object):
                 include_symbol = include_symbol
             )
 
-         .. versionadded:: 0.3.6
+         .. seealso::
+
+            :paramref:`.EnvironmentContext.configure.include_schemas`
+
+            :paramref:`.EnvironmentContext.configure.include_object`
+
+        :param include_schemas: If True, autogenerate will scan across
+         all schemas located by the SQLAlchemy
+         :meth:`~sqlalchemy.engine.reflection.Inspector.get_schema_names`
+         method, and include all differences in tables found across all
+         those schemas.  When using this option, you may want to also
+         use the :paramref:`.EnvironmentContext.configure.include_object`
+         option to specify a callable which
+         can filter the tables/schemas that get included.
+
+         .. versionadded :: 0.4.0
+
+         .. seealso::
+
+            :paramref:`.EnvironmentContext.configure.include_object`
+
+        :param render_item: Callable that can be used to override how
+         any schema item, i.e. column, constraint, type,
+         etc., is rendered for autogenerate.  The callable receives a
+         string describing the type of object, the object, and
+         the autogen context.  If it returns False, the
+         default rendering method will be used.  If it returns None,
+         the item will not be rendered in the context of a Table
+         construct, that is, can be used to skip columns or constraints
+         within op.create_table()::
+
+            def my_render_column(type_, col, autogen_context):
+                if type_ == "column" and isinstance(col, MySpecialCol):
+                    return repr(col)
+                else:
+                    return False
+
+            context.configure(
+                # ...
+                render_item = my_render_column
+            )
+
+         Available values for the type string include: ``"column"``,
+         ``"primary_key"``, ``"foreign_key"``, ``"unique"``, ``"check"``,
+         ``"type"``, ``"server_default"``.
+
+         .. versionadded:: 0.5.0
+
+         .. seealso::
+
+            :ref:`autogen_render_types`
 
         :param upgrade_token: When autogenerate completes, the text of the
          candidate upgrade operations will be present in this template
@@ -395,6 +589,21 @@ class EnvironmentContext(object):
          will render them using the dialect module name, i.e. ``mssql.BIT()``,
          ``postgresql.UUID()``.
 
+        :param user_module_prefix: When autogenerate refers to a SQLAlchemy
+         type (e.g. :class:`.TypeEngine`) where the module name is not
+         under the ``sqlalchemy`` namespace, this prefix will be used
+         within autogenerate, if non-``None``; if left at its default of
+         ``None``, the
+         :paramref:`.EnvironmentContext.configure.sqlalchemy_module_prefix`
+         is used instead.
+
+         .. versionadded:: 0.6.3 added
+            :paramref:`.EnvironmentContext.configure.user_module_prefix`
+
+         .. seealso::
+
+            :ref:`autogen_module_prefix`
+
         Parameters specific to individual backends:
 
         :param mssql_batch_separator: The "batch separator" which will
@@ -412,7 +621,7 @@ class EnvironmentContext(object):
         """
         opts = self.context_opts
         if transactional_ddl is not None:
-            opts["transactional_ddl"] =  transactional_ddl
+            opts["transactional_ddl"] = transactional_ddl
         if output_buffer is not None:
             opts["output_buffer"] = output_buffer
         elif self.config.output_buffer is not None:
@@ -425,10 +634,15 @@ class EnvironmentContext(object):
             opts['template_args'].update(template_args)
         opts['target_metadata'] = target_metadata
         opts['include_symbol'] = include_symbol
+        opts['include_object'] = include_object
+        opts['include_schemas'] = include_schemas
         opts['upgrade_token'] = upgrade_token
         opts['downgrade_token'] = downgrade_token
         opts['sqlalchemy_module_prefix'] = sqlalchemy_module_prefix
         opts['alembic_module_prefix'] = alembic_module_prefix
+        opts['user_module_prefix'] = user_module_prefix
+        if render_item is not None:
+            opts['render_item'] = render_item
         if compare_type is not None:
             opts['compare_type'] = compare_type
         if compare_server_default is not None:
@@ -520,9 +734,9 @@ class EnvironmentContext(object):
           the output stream, as rendered by the
           target backend (e.g. SQL Server would
           emit ``BEGIN TRANSACTION``).
-        * Otherwise, calls :meth:`sqlalchemy.engine.base.Connection.begin`
+        * Otherwise, calls :meth:`sqlalchemy.engine.Connection.begin`
           on the current online connection, which
-          returns a :class:`sqlalchemy.engine.base.Transaction`
+          returns a :class:`sqlalchemy.engine.Transaction`
           object.  This object demarcates a real
           transaction and is itself a context manager,
           which will roll back if an exception
@@ -530,7 +744,7 @@ class EnvironmentContext(object):
 
         Note that a custom ``env.py`` script which
         has more specific transactional needs can of course
-        manipulate the :class:`~sqlalchemy.engine.base.Connection`
+        manipulate the :class:`~sqlalchemy.engine.Connection`
         directly to produce transactional state in "online"
         mode.
 
@@ -566,7 +780,7 @@ class EnvironmentContext(object):
         """Return the current 'bind'.
 
         In "online" mode, this is the
-        :class:`sqlalchemy.engine.base.Connection` currently being used
+        :class:`sqlalchemy.engine.Connection` currently being used
         to emit SQL to the database.
 
         This function requires that a :class:`.MigrationContext`
