@@ -9,6 +9,11 @@ from .ddl import impl
 
 __all__ = ('Operations',)
 
+try:
+    from sqlalchemy.sql.naming import conv
+except:
+    conv = None
+
 class Operations(object):
     """Define high level migration operations.
 
@@ -55,7 +60,7 @@ class Operations(object):
 
 
     def _primary_key_constraint(self, name, table_name, cols, schema=None):
-        m = sa_schema.MetaData()
+        m = self._metadata()
         columns = [sa_schema.Column(n, NULLTYPE) for n in cols]
         t1 = sa_schema.Table(table_name, m,
                 *columns,
@@ -68,8 +73,9 @@ class Operations(object):
                                     local_cols, remote_cols,
                                     onupdate=None, ondelete=None,
                                     deferrable=None, source_schema=None,
-                                    referent_schema=None):
-        m = sa_schema.MetaData()
+                                    referent_schema=None, initially=None,
+                                    match=None, **dialect_kw):
+        m = self._metadata()
         if source == referent:
             t1_cols = local_cols + remote_cols
         else:
@@ -90,14 +96,17 @@ class Operations(object):
                                             name=name,
                                             onupdate=onupdate,
                                             ondelete=ondelete,
-                                            deferrable=deferrable
+                                            deferrable=deferrable,
+                                            initially=initially,
+                                            match=match,
+                                            **dialect_kw
                                             )
         t1.append_constraint(f)
 
         return f
 
     def _unique_constraint(self, name, source, local_cols, schema=None, **kw):
-        t = sa_schema.Table(source, sa_schema.MetaData(),
+        t = sa_schema.Table(source, self._metadata(),
                     *[sa_schema.Column(n, NULLTYPE) for n in local_cols],
                     schema=schema)
         kw['name'] = name
@@ -108,14 +117,22 @@ class Operations(object):
         return uq
 
     def _check_constraint(self, name, source, condition, schema=None, **kw):
-        t = sa_schema.Table(source, sa_schema.MetaData(),
+        t = sa_schema.Table(source, self._metadata(),
                     sa_schema.Column('x', Integer), schema=schema)
         ck = sa_schema.CheckConstraint(condition, name=name, **kw)
         t.append_constraint(ck)
         return ck
 
+    def _metadata(self):
+        kw = {}
+        if 'target_metadata' in self.migration_context.opts:
+            mt = self.migration_context.opts['target_metadata']
+            if hasattr(mt, 'naming_convention'):
+                kw['naming_convention'] = mt.naming_convention
+        return sa_schema.MetaData(**kw)
+
     def _table(self, name, *columns, **kw):
-        m = sa_schema.MetaData()
+        m = self._metadata()
         t = sa_schema.Table(name, m, *columns, **kw)
         for f in t.foreign_keys:
             self._ensure_table_for_fk(m, f)
@@ -125,7 +142,7 @@ class Operations(object):
         return sa_schema.Column(name, type_, **kw)
 
     def _index(self, name, tablename, columns, schema=None, **kw):
-        t = sa_schema.Table(tablename or 'no_table', sa_schema.MetaData(),
+        t = sa_schema.Table(tablename or 'no_table', self._metadata(),
             *[sa_schema.Column(n, NULLTYPE) for n in columns],
             schema=schema
         )
@@ -309,6 +326,52 @@ class Operations(object):
                 if _count_constraint(constraint):
                     self.impl.add_constraint(constraint)
 
+    def f(self, name):
+        """Indicate a string name that has already had a naming convention
+        applied to it.
+
+        This feature combines with the SQLAlchemy ``naming_convention`` feature
+        to disambiguate constraint names that have already had naming
+        conventions applied to them, versus those that have not.  This is
+        necessary in the case that the ``"%(constraint_name)s"`` token
+        is used within a naming convention, so that it can be identified
+        that this particular name should remain fixed.
+
+        If the :meth:`.Operations.f` is used on a constraint, the naming
+        convention will not take effect::
+
+            op.add_column('t', 'x', Boolean(name=op.f('ck_bool_t_x')))
+
+        Above, the CHECK constraint generated will have the name ``ck_bool_t_x``
+        regardless of whether or not a naming convention is in use.
+
+        Alternatively, if a naming convention is in use, and 'f' is not used,
+        names will be converted along conventions.  If the ``target_metadata``
+        contains the naming convention
+        ``{"ck": "ck_bool_%(table_name)s_%(constraint_name)s"}``, then the
+        output of the following:
+
+            op.add_column('t', 'x', Boolean(name='x'))
+
+        will be::
+
+            CONSTRAINT ck_bool_t_x CHECK (x in (1, 0)))
+
+        The function is rendered in the output of autogenerate when
+        a particular constraint name is already converted, for SQLAlchemy
+        version **0.9.4 and greater only**.   Even though ``naming_convention``
+        was introduced in 0.9.2, the string disambiguation service is new
+        as of 0.9.4.
+
+        .. versionadded:: 0.6.4
+
+        """
+        if conv:
+            return conv(name)
+        else:
+            raise NotImplementedError(
+                    "op.f() feature requires SQLAlchemy 0.9.4 or greater.")
+
     def add_column(self, table_name, column, schema=None):
         """Issue an "add column" instruction using the current
         migration context.
@@ -394,6 +457,17 @@ class Operations(object):
          SQL-script-compatible
          block that selects into a @variable from sys.default_constraints,
          then exec's a separate DROP CONSTRAINT for that default.
+        :param mssql_drop_foreign_key: Optional boolean.  When ``True``, on
+         Microsoft SQL Server only, first
+         drop a single FOREIGN KEY constraint on the column using a
+         SQL-script-compatible
+         block that selects into a @variable from
+         sys.foreign_keys/sys.foreign_key_columns,
+         then exec's a separate DROP CONSTRAINT for that default.  Only
+         works if the column has exactly one FK constraint which refers to
+         it, at the moment.
+
+         .. versionadded:: 0.6.2
 
         """
 
@@ -444,10 +518,12 @@ class Operations(object):
                                 schema)
                 )
 
+
     def create_foreign_key(self, name, source, referent, local_cols,
                            remote_cols, onupdate=None, ondelete=None,
-                           deferrable=None, source_schema=None,
-                           referent_schema=None):
+                           deferrable=None, initially=None, match=None,
+                           source_schema=None, referent_schema=None,
+                           **dialect_kw):
         """Issue a "create foreign key" instruction using the
         current migration context.
 
@@ -497,7 +573,8 @@ class Operations(object):
                             local_cols, remote_cols,
                             onupdate=onupdate, ondelete=ondelete,
                             deferrable=deferrable, source_schema=source_schema,
-                            referent_schema=referent_schema)
+                            referent_schema=referent_schema,
+                            initially=initially, match=match, **dialect_kw)
                 )
 
     def create_unique_constraint(self, name, source, local_cols,
@@ -763,7 +840,7 @@ class Operations(object):
         t.append_constraint(const)
         self.impl.drop_constraint(const)
 
-    def bulk_insert(self, table, rows):
+    def bulk_insert(self, table, rows, multiinsert=True):
         """Issue a "bulk insert" operation using the current
         migration context.
 
@@ -797,8 +874,55 @@ class Operations(object):
                             'create_date':date(2008, 8, 15)},
                 ]
             )
+
+        When using --sql mode, some datatypes may not render inline automatically,
+        such as dates and other special types.   When this issue is present,
+        :meth:`.Operations.inline_literal` may be used::
+
+            op.bulk_insert(accounts_table,
+                [
+                    {'id':1, 'name':'John Smith',
+                            'create_date':op.inline_literal("2010-10-05")},
+                    {'id':2, 'name':'Ed Williams',
+                            'create_date':op.inline_literal("2007-05-27")},
+                    {'id':3, 'name':'Wendy Jones',
+                            'create_date':op.inline_literal("2008-08-15")},
+                ],
+                multiinsert=False
+            )
+
+        When using :meth:`.Operations.inline_literal` in conjunction with
+        :meth:`.Operations.bulk_insert`, in order for the statement to work
+        in "online" (e.g. non --sql) mode, the
+        :paramref:`~.Operations.bulk_insert.multiinsert`
+        flag should be set to ``False``, which will have the effect of
+        individual INSERT statements being emitted to the database, each
+        with a distinct VALUES clause, so that the "inline" values can
+        still be rendered, rather than attempting to pass the values
+        as bound parameters.
+
+        .. versionadded:: 0.6.4 :meth:`.Operations.inline_literal` can now
+           be used with :meth:`.Operations.bulk_insert`, and the
+           :paramref:`~.Operations.bulk_insert.multiinsert` flag has
+           been added to assist in this usage when running in "online"
+           mode.
+
+        :param table: a table object which represents the target of the INSERT.
+
+        :param rows: a list of dictionaries indicating rows.
+
+        :param multiinsert: when at its default of True and --sql mode is not
+           enabled, the INSERT statement will be executed using
+           "executemany()" style, where all elements in the list of dictionaries
+           are passed as bound parameters in a single list.   Setting this
+           to False results in individual INSERT statements being emitted
+           per parameter set, and is needed in those cases where non-literal
+           values are present in the parameter sets.
+
+           .. versionadded:: 0.6.4
+
           """
-        self.impl.bulk_insert(table, rows)
+        self.impl.bulk_insert(table, rows, multiinsert=multiinsert)
 
     def inline_literal(self, value, type_=None):
         """Produce an 'inline literal' expression, suitable for
