@@ -7,7 +7,7 @@ import textwrap
 
 from nose import SkipTest
 from sqlalchemy.engine import default
-from sqlalchemy import create_engine, text
+from sqlalchemy import create_engine, text, MetaData
 from sqlalchemy.exc import SQLAlchemyError
 from sqlalchemy.util import decorator
 
@@ -29,9 +29,11 @@ testing_config.read(['test.cfg'])
 
 if py33:
     from unittest.mock import Mock, call
+    from unittest import mock
 else:
     try:
         from mock import Mock, call
+        import mock
     except ImportError:
         raise ImportError(
                 "Alembic's test suite requires the "
@@ -91,6 +93,12 @@ def requires_092(fn, *arg, **kw):
         raise SkipTest("SQLAlchemy 0.9.2 or greater required")
     return fn(*arg, **kw)
 
+@decorator
+def requires_094(fn, *arg, **kw):
+    if not util.sqla_094:
+        raise SkipTest("SQLAlchemy 0.9.4 or greater required")
+    return fn(*arg, **kw)
+
 _dialects = {}
 def _get_dialect(name):
     if name is None or name == 'default':
@@ -160,7 +168,7 @@ def assert_raises_message(except_cls, msg, callable_, *args, **kwargs):
         assert re.search(msg, str(e)), "%r !~ %s" % (msg, e)
         print(text_type(e))
 
-def op_fixture(dialect='default', as_sql=False):
+def op_fixture(dialect='default', as_sql=False, naming_convention=None):
     impl = _impls[dialect]
     class Impl(impl):
         def __init__(self, dialect, as_sql):
@@ -174,18 +182,26 @@ def op_fixture(dialect='default', as_sql=False):
         def _exec(self, construct, *args, **kw):
             if isinstance(construct, string_types):
                 construct = text(construct)
+            assert construct.supports_execution
             sql = text_type(construct.compile(dialect=self.dialect))
             sql = re.sub(r'[\n\t]', '', sql)
             self.assertion.append(
                 sql
             )
 
+    opts = {}
+    if naming_convention:
+        if not util.sqla_092:
+            raise SkipTest(
+                        "naming_convention feature requires "
+                        "sqla 0.9.2 or greater")
+        opts['target_metadata'] = MetaData(naming_convention=naming_convention)
 
     class ctx(MigrationContext):
         def __init__(self, dialect='default', as_sql=False):
             self.dialect = _get_dialect(dialect)
             self.impl = Impl(self.dialect, as_sql)
-
+            self.opts = opts
             self.as_sql = as_sql
 
         def assert_(self, *sql):
@@ -228,12 +244,13 @@ config = context.config
     with open(path, 'w') as f:
         f.write(txt)
 
-def _sqlite_testing_config():
+def _sqlite_testing_config(sourceless=False):
     dir_ = os.path.join(staging_directory, 'scripts')
     return _write_config_file("""
 [alembic]
 script_location = %s
 sqlalchemy.url = sqlite:///%s/foo.db
+sourceless = %s
 
 [loggers]
 keys = root
@@ -258,7 +275,7 @@ keys = generic
 [formatter_generic]
 format = %%(levelname)-5.5s [%%(name)s] %%(message)s
 datefmt = %%H:%%M:%%S
-    """ % (dir_, dir_))
+    """ % (dir_, dir_, "true" if sourceless else "false"))
 
 
 def _no_sql_testing_config(dialect="postgresql", directives=""):
@@ -349,7 +366,7 @@ def write_script(scriptdir, rev_id, content, encoding='ascii', sourceless=False)
     pyc_path = util.pyc_file_from_path(path)
     if os.access(pyc_path, os.F_OK):
         os.unlink(pyc_path)
-    script = Script._from_path(path)
+    script = Script._from_path(scriptdir, path)
     old = scriptdir._revision_map[script.revision]
     if old.down_revision != script.down_revision:
         raise Exception("Can't change down_revision "
